// (c) 2008 Elli Computing
// Name: _facilities.js
// Version: 1.1
// Purpose: Helpful functions for macros

// returns -1, 0, 1 respectively if version ver1 is before, at or after ver2
// version strings should be formatted is "major.minor.build" (as Application.version)
function ec_compare_ec_version (ver1, ver2)
{
	var verarray1 = ver1.split(".");
	var verarray2 = ver2.split(".");
	if (verarray1.length != 3 || verarray2.length != 3)
		throw "Invalid version string";
		
	for (var i=0; i<3; i++)
		if (verarray1[i] < verarray2[i]) return -1;
		else if (verarray1[i] > verarray2[i]) return 1;
		
	return 0;
}

// returns true if current version is after or at <startver>
// if <endver> is specified current version must also be before it
function ec_check_version (startver, endver)
{
	return ec_compare_ec_version(startver, Application.version) <= 0 &&
		(!endver || ec_compare_ec_version(endver, Application.version) > 0);
}

// verifies that a array is of current script generation, if it is not the case the array is replaced by []
function ec_check_array_script_generation (array_name, base)
{
	if (!base.hasOwnProperty(array_name) ||
		base[array_name].scripts_generation != Application.scripts_generation)
	{
		base[array_name] = [ ];
		base[array_name].scripts_generation = Application.scripts_generation;
	}
}

// returns a pair { found_at, found }, if <found> is false, <found_at> is the
//	place where the object was expected to be [therefore where to insert it...]
// if <start> or <end> is undefined/ommitted, start==0 and end=collection.length
function ec_binary_search (collection, comparator, searched, start, _end)
{
	if (start === undefined) start = 0;
	if (_end === undefined) _end = collection.length;
	
	if (start == _end) return { found_at: start, found: false };
	
	var beg=start, end=_end-1, mid;
	var	diff;

	while (end > beg)
	{
		mid = (beg+end)>>1;	// was /2 but beloved ECMASCript may return x.5 positions...
		// log (mid);

		diff = comparator(collection[mid], searched);

		if (diff < 0)		// <mid> element is strictly before <elem>
		{
			beg = mid+1;
		}
		else if (diff > 0)
		{
			end = mid-1;
		}
		else
		{
			return { found_at: mid, found: true };
		}
	}

	// I must verify that <beg> is not the value... if not, I did not find it
	// log (beg);

	diff = comparator (collection[beg], searched);
	if (diff < 0)
		++beg;
	return { found_at: beg, found: diff == 0 };
}

//////////////////////////////////////////////////////////////////////////////
// MRU handling

// a MRU is a simple Array object

// this function adds an entry to the top of the MRU
// if you provide a 'equality_test', the first previously existing entry 
//	for which it returns true gets removed
// the new entry is added to the top of the array
// if you don't provide a 'mru_size', a default size of 16 is used
// if the Array length is more than the MRU size, the Array is shortened
function ec_add_to_mru (entry, mru, mru_size, equality_test)
{
	if (equality_test === undefined) equality_test = function(x,y) { return x==y; }
	if (mru_size === undefined) mru_size = 16;
	
	for (var idx=0; idx<mru.length; ++idx)
		if (equality_test(entry, mru[idx]))
		{
			mru.splice (idx, 1);
			break;
		}
		
	mru.splice (0, 0, entry);
	
	if (mru.length > mru_size)
		mru.length = mru_size;
}

// automaticly prompt with choices if choices are available
// takes a choices list from the scripts settings 'setting_name'
function ec_prompt_with_choices(prompt_text, setting_name)
{
	if (Application.scripts_settings)
	{
		var entered_text;
		var choices = Application.scripts_settings[setting_name];
		if (choices)
		{
			entered_text = prompt_with_choices (prompt_text, choices);
		}
		else
		{
			entered_text = prompt(prompt_text);
			choices = [];
		}
		if (entered_text)
		{
			ec_add_to_mru (entered_text, choices);
			Application.scripts_settings[setting_name] = choices;				
		}
		
		return entered_text;
	}
	else
		return prompt(prompt_text);
}

// returns the value stored in the settings or the default value if it is not there at all
function ec_get_setting_or_default (setting_name, default_value)
{
    if (Application.scripts_settings &&
        Application.scripts_settings[setting_name] instanceof Array &&
        Application.scripts_settings[setting_name].length > 0)
        return Application.scripts_settings[setting_name][0];
    else
        return default_value;
}

//////////////////////////////////////////////////////////////////////////////
// INTERNATIONALIZATION (I18N)

// the '_' function is a 'standard gettext' notation for the function which
//	translates messages in user language
function _(a,b,c,d)
{
	return I18N.translate(a,b,c,d);
}

//////////////////////////////////////////////////////////////////////////////
// HANDLES SHARED/DOCUMENT TYPE CONTEXTUAL MENU 

if (Document.View !== undefined)
	Document.View.prototype.query_contextual_menu = function () {
		var menu = [];
		
		if (Document.View.prototype.contextual_menu_items)
		{
			for (var i=0; i<Document.View.prototype.contextual_menu_items.length; ++i)
			{
				menu.push (Document.View.prototype.contextual_menu_items[i]);
			}
		}
		
		if (this.contextual_menu_items && this.contextual_menu_items !== Document.View.prototype.contextual_menu_items)
		{
			if (menu.length != 0)
				menu.push ( { label: "" } );
				
			for (var i=0; i<this.contextual_menu_items.length; ++i)
			{
				menu.push (this.contextual_menu_items[i]);
			}
		}
		
		return menu;
	}



