// (c) Elli Computing 2008-2012
// this script expects an array 'ec_converters_detectors' to be present in the
//	global object (it does nothing if not found, but make an error
//	if it is not an Array), this array should contain functions
//	taking an OptionSet object as parameter, in which the detector can
//	check if the converter it handles already exists, and add it 
//	with wise parameters if it is not yet

// Anatomy of detector:
// - each detector function must first check if its converter is already added or not
//		- if it is not there, it should add the converter even if the tool is not found
//			the tool should be set as 'active'
//		- whether there or not, the path is updated (the user must copy the converter
//          and de-atcivate the original if s/he wants to customize it)
//		- if it is there, it should fill the path, but not set the tool as active

function ec_detect_converters (option_set)
{
	if (typeof ec_converters_detectors == 'undefined')
		return;
	if (ec_converters_detectors.constructor !== Array)
		throw Error('ec_converters_detectors is not an Array');
		
	for (converter_detector in ec_converters_detectors)
	{
		if (converter_detector == 'scripts_generation') continue;
		
		try {
			ec_converters_detectors[converter_detector] (option_set);
		}
		catch (e) {
			log ("Converter detection function "+converter_detector+ "failed with error: "+e);
		}
	}
}

function ec_find_converter (option_set, name)
{
	return option_set["filters.loading"].find_index ('name', name);
}

// returns the local path of the executable which takes: path yes/no in yes/no out args
//	as parameters (path executable to call with in as stdin (if preceded by yes)
//	 and out as stdout (if preceded by yes))
function ec_get_stdio_converter()
{
	var stdioconv;
	if (Machine.system_name.match (/.*Win.*/))
		if (Machine.system_name.match (/.*(9[58]|Me).*/))
			stdioconv = "stdioconv9x.bat";
		else
			stdioconv = "stdioconv.bat";
	else
		stdioconv = "stdioconv.sh";
	
	return String(URL(Application.executable_path).compose("../converters/"+stdioconv));
}

function ec_get_default_app_extension()
{
	if (Machine.system_name.match (/.*Win.*/))
		return '.exe';
	else
		return '';
}

function ec_item_exists_in_dir (url, file_name)
{
    function escapeRegExp(string) {
      return string.replace(/([.*+?^=!:${}()|[\]\/\\])/g, "\\$1");
    }

    try {
        return VFS.dir(url, new RegExp(escapeRegExp(file_name), "i")).length != 0;
    }
    catch (e) {
        return false;
    }
}

function ecmerge_converters_which(executable_name)
{
    var exec_dir_url = URL(executable_path).compose("..");
    if (ec_item_exists_in_dir (exec_dir_url.compose("user-converters"), executable_name))
        return exec_dir_url.compose("user-converters/"+executable_name);

    if (ec_item_exists_in_dir (exec_dir_url.compose("converters"), executable_name))
        return exec_dir_url.compose("converters/"+executable_name);

    return Machine.which(executable_name);
}

var ec_converters_detectors;
if (ec_converters_detectors === undefined ||
	ec_converters_detectors.scripts_generation === undefined ||
	ec_converters_detectors.scripts_generation != Application.scripts_generation)
{
	ec_converters_detectors = [];
	ec_converters_detectors.scripts_generation = Application.scripts_generation;
}

////////////////////////////////////////////////////////////////////
// Archives converters

function ec_7zip_converter(option_set)
{
	var converter_name = "7 Zip"; // have a variable for that to be sure, find/new share the same name
    var converter;
	var found_converter = ec_find_converter(option_set, converter_name);
    
    if (found_converter===undefined)
	{
		var converter = new UserProcessFilter;
		converter.name = converter_name;
        found_converter = option_set["filters.loading"].length;
	}
    else
        converter = option_set["filters.loading"][found_converter].clone();

    converter.arguments = "x -o$(output) $(input)";
	converter.output_is_folder = true;
	converter.activate_by_pattern = true;

	converter.activation_pattern = "*.7z;*.zip;*.tar;*.tgz;*.tbz;*.rar;*.cab;*.arj;*.rpm;*.deb;*.chm";
    converter.activation_document_type = "folders";

    if (converter.application_path == "")
    {
        var path;
        if (Machine.system_name.match (/.*Win.*/) &&
		        (path=Registry.get_value ("HKEY_LOCAL_MACHINE\\SOFTWARE\\7-Zip", "Path")) !== undefined)
	        path = path + "\\7z.exe";
        else
	        path=ecmerge_converters_which ("7z" + ec_get_default_app_extension());
		
        if (path !== undefined)
            converter.application_path = path;
    }
    option_set["filters.loading"][found_converter] = converter;
}

ec_converters_detectors.push(ec_7zip_converter);

function ec_add_zfilter (option_set, converter_name, patterns, executable_name, redir_in, redir_out, args, output_ext, activate_as_sub_doc)
{
	var found_converter = ec_find_converter(option_set, converter_name);
    
    if (found_converter===undefined)
	{
//		alert(converter_name);
		var converter = new UserProcessFilter;
		converter.name = converter_name;
		converter.output_is_folder = false;
		converter.activate_by_pattern = true;
        found_converter = option_set["filters.loading"].length;
	}
    else
        converter = option_set["filters.loading"][found_converter].clone();

	converter.activation_pattern = patterns;
	converter.activate_as_sub_doc = Boolean(activate_as_sub_doc);

	if (output_ext !== undefined)
		converter.output_extension = output_ext;
    else
        converter.output_extension = "";

    if (!redir_in && !redir_out)
        converter.arguments = args;

    if (converter.application_path == "")
    {
		var path;
//		alert(executable_name);
		if ((path=ecmerge_converters_which(executable_name + ec_get_default_app_extension())) !== undefined)
		{
//			alert(path);
			if (redir_in || redir_out)
			{
				converter.application_path = ec_get_stdio_converter();
				converter.arguments = Machine.escape_for_command_line(path) + " "
					+ (redir_in ? "yes" : "no") + "  $(input) " + (redir_out ? "yes" : "no") +
					" $(output) " + args;
			}
			else
			{
				converter.application_path = path;
			}
        }
    }
    option_set["filters.loading"][found_converter] = converter;
}

function ec_gzip_converter(option_set)
{
	return ec_add_zfilter (option_set, "GZip", "*.gz;*.gzip", "gzip", true, true, "-cdf");
}

ec_converters_detectors.push(ec_gzip_converter);

function ec_bzip_converter(option_set)
{
	return ec_add_zfilter (option_set, "BZip2", "*.bz;*.bzip;*.bz2", "bzip2", false, true, "-cdfk $(input)");
}

ec_converters_detectors.push(ec_bzip_converter);

function ec_winzipcl_converter(option_set)
{
	var converter;
	var converter_name = "WinZip Command Line"; // have a variable for that to be sure, find/new share the same name
	var found_converter = ec_find_converter(option_set, converter_name);
	if (found_converter===undefined)
	{
		converter = new UserProcessFilter;
		converter.name = converter_name;
		converter.output_is_folder = true;
		converter.activate_by_pattern = true;

		converter.activation_pattern = "*.7z;*.cab;*.hqx;*.img;*.iso;*.kha;*.lzh;*.rar;*.tar;*.taz;*.tbz;*.tbz2;*.tgz;*.tz;*.zip";
		converter.arguments = "-d -ybc $(input) $(output)";
		
		found_converter = option_set["filters.loading"].length;
	}
	else
		converter = option_set["filters.loading"][found_converter].clone();

    converter.activation_document_type = "folders";
	
	if (converter.application_path == "")
	{
		var path;
		if (Machine.system_name.match (/.*Win.*/) &&
			 (path=Registry.get_value ("HKEY_CLASSES_ROOT\\WinZip\\shell\\open\\command", "")) !== undefined)
			path = URL(path).compose("../wzunzip.exe")
		else
			path=ecmerge_converters_which ("wzunzip" + ec_get_default_app_extension());
		
		if (path !== undefined)
			converter.application_path = path;
	}
	
	option_set["filters.loading"][found_converter] = converter;
}

ec_converters_detectors.push(ec_winzipcl_converter);

function ec_winrar_dir_converter(option_set)
{
	var converter;
	var converter_name = "WinRAR"; // have a variable for that to be sure, find/new share the same name
	var found_converter = ec_find_converter(option_set, converter_name);
	if (found_converter===undefined)
	{
		converter = new UserProcessFilter;
		converter.name = converter_name;
		converter.output_is_folder = true;
		converter.activate_by_pattern = true;

		converter.activation_pattern = "*.ace;*.arj;*.7z;*.cab;*.lzh;*.rar;*.tar;*.taz;*.tbz;*.tbz2;*.tgz;*.tz;*.zip";
		
		found_converter = option_set["filters.loading"].length;
	}
	else
		converter = option_set["filters.loading"][found_converter].clone();

	converter.arguments = "x $(input) *.* \"$(nake-output)\\\"";
    converter.activation_document_type = "folders";
	
	if (converter.application_path == "")
	{
		var path;
		if (Machine.system_name.match (/.*Win.*/) &&
			 (path=Registry.get_value ("HKEY_CLASSES_ROOT\\.rar\\ShellNew", "FileName")) !== undefined)
			path = URL(path).compose("../winrar.exe")
		else
			path=ecmerge_converters_which ("winrar" + ec_get_default_app_extension());
		
		if (path !== undefined)
			converter.application_path = path;
	}
	
	option_set["filters.loading"][found_converter] = converter;
}

ec_converters_detectors.push(ec_winrar_dir_converter);

//////////////////////////////////////////////////////////////////
// Imaging converters

function ec_imagemagick_converter(option_set)
{
	var path;
	if (Machine.system_name.match (/.*Win.*/) &&
		(path=Registry.get_value ("HKEY_LOCAL_MACHINE\\SOFTWARE\\ImageMagick\\Current", "BinPath")) !== undefined)
		path = path + "\\convert.exe";
	else
	{
		path = ecmerge_converters_which ("convert" + ec_get_default_app_extension());
		
		// accept that path only if there is "image" and "magick" somewhere in the path
		if (! /image.*magick/i.test (path) && ! /converters/i.test(path))
			path = undefined;
	}
		
	var converter_name = "Image Magick"; // have a variable for that to be sure, find/new share the same name
    var found_converter = ec_find_converter(option_set, converter_name);
	if (found_converter === undefined)
	{
		var converter = new UserProcessFilter;
		converter.name = converter_name;
		converter.output_is_folder = false;
		converter.activate_by_pattern = true;
		converter.activation_document_type = "images";
		converter.output_extension = ".bmp";
		converter.activate_as_sub_doc = true;

		converter.activation_pattern = "*.psd;*.cur;*.art;*.avs;*.crw;*.cut;*.dcm;*.dcr;*.dcx;*.djvu;*.fax;*.mono;*.mrw;*.mtv;*.nef;*.orf;*.otb;*.p7;*.palm;*.pbm;*.pcd;*.pcds;*.pcl;*.pcx;"+
			"*.pfa;*.pfb;*.pfm;*.pgm;*.picon;*.pict;*.pix;*.ppm;*.pwp;*.raf;*.rla;*.rle;*.sct;*.sfw;*.sgi;*.sun;*.icb;*.vda;*.tga;*.vst;*.tim;*.vicar;*.wpg;*.xbm;*.xcf;*.xpm;*.xwd;*.x3f";

        found_converter = option_set["filters.loading"].length;
    }
    else
		converter = option_set["filters.loading"][found_converter].clone();

	converter.arguments = "$(input) $(output)";

    if (converter.application_path == "" && path !== undefined)
		converter.application_path = path;

	option_set["filters.loading"][found_converter] = converter;

	converter_name = "Image Magick Vectored"; // have a variable for that to be sure, find/new share the same name
    found_converter = ec_find_converter(option_set, converter_name);
	if (found_converter === undefined)
	{
		var converter = new UserProcessFilter;
		converter.name = converter_name;
		converter.output_is_folder = false;
		converter.activate_by_pattern = true;
		converter.activation_document_type = "images";
		converter.output_extension = ".bmp";
		converter.activate_as_sub_doc = true;

		converter.activation_pattern = "*.svg;*.eps";

		found_converter = option_set["filters.loading"].length;
    }
    else
		converter = option_set["filters.loading"][found_converter].clone();

	converter.arguments = "-size 500x500 $(input) $(output)";

	if (converter.application_path == "" && path !== undefined)
		converter.application_path = path;

	option_set["filters.loading"][found_converter] = converter;
}

ec_converters_detectors.push(ec_imagemagick_converter);


function ec_dcraw_converter(option_set)
{
	var converter_name = "dcraw"; // have a variable for that to be sure, find/new share the same name
	
	ec_add_zfilter (option_set, converter_name, "*.dng;*.crw;*.cr2;*.mrw;*.nef;*.raf",
		(Machine.system_name.match (/.*Win.*/) ? "dcrawms" : "dcraw"), false, true, "-T -c $(input)", ".tif", true);
}

ec_converters_detectors.push(ec_dcraw_converter);

//////////////////////////////////////////////////////////////////
// PDF converters

function ec_pdf_converter(option_set)
{
	var converter_name = "PDF to Text (xpdf)"; // have a variable for that to be sure, find/new share the same name
    var found_converter = ec_find_converter(option_set, converter_name);

	if (found_converter === undefined)
	{
		var converter = new UserProcessFilter;
		converter.name = converter_name;
		converter.output_is_folder = false;
		converter.activate_as_sub_doc = true;
		converter.activate_by_pattern = true;
		converter.activation_document_type = "texts";
		converter.output_extension = ".txt";

		converter.activation_pattern = "*.pdf";
        found_converter = option_set["filters.loading"].length;
    }
    else
		converter = option_set["filters.loading"][found_converter].clone();

    converter.arguments = "-enc UTF-8 -nopgbrk -q $(input) $(output)";

	if (converter.application_path == "")
    {
		var path;
		if ((path=ecmerge_converters_which("pdftotext" + ec_get_default_app_extension())) !== undefined)
		{
			converter.application_path = path;
        }
    }
    option_set["filters.loading"][found_converter] = converter;
	
	converter_name = "PDF to Text (Intelligent Converters)"; // have a variable for that to be sure, find/new share the same name
	found_converter = ec_find_converter(option_set, converter_name);

	if (found_converter === undefined)
	{
		var converter = new UserProcessFilter;
		converter.name = converter_name;
		converter.output_is_folder = false;
		converter.activate_as_sub_doc = true;
		converter.activate_by_pattern = true;
		converter.activation_document_type = "texts";
		converter.output_extension = ".txt";

		converter.activation_pattern = "*.pdf";
        found_converter = option_set["filters.loading"].length;
	}
    else
        converter = option_set["filters.loading"][found_converter].clone();

	converter.arguments = "--src=$(input) --dest=$(output)";

    if (converter.application_path == "")
    {
		var path;
		if (Machine.system_name.match (/.*Win.*/) &&
			(path=ecmerge_converters_which("p2tagent" + ec_get_default_app_extension())) !== undefined)
		{
			converter.application_path = path;
        }
    }
    option_set["filters.loading"][found_converter] = converter;
	
	converter_name = "PDF to Text (VeryPDF)"; // have a variable for that to be sure, find/new share the same name
	found_converter = ec_find_converter(option_set, converter_name);
    
    if (found_converter===undefined)
	{
		var converter = new UserProcessFilter;
		converter.name = converter_name;
		converter.output_is_folder = false;
		converter.activate_as_sub_doc = true;
		converter.activate_by_pattern = true;
		converter.activation_document_type = "texts";
		converter.output_extension = ".txt";

		converter.activation_pattern = "*.pdf";
        found_converter = option_set["filters.loading"].length;
	}
    else
        converter = option_set["filters.loading"][found_converter].clone();

	converter.arguments = "$(input) $(output) -unicode -silent";

    if (converter.application_path == "")
    {
		var path;
		if (Machine.system_name.match (/.*Win.*/) &&
			(path=ecmerge_converters_which("pdf2txt" + ec_get_default_app_extension())) !== undefined)
		{
			converter.application_path = path;
        }
    }
    option_set["filters.loading"][found_converter] = converter;
	
	converter_name = "PDF Text Converter (DigitZone)"; // have a variable for that to be sure, find/new share the same name
	found_converter = ec_find_converter(option_set, converter_name);
    
    if (found_converter===undefined)
	{
		var converter = new UserProcessFilter;
		converter.name = converter_name;
		converter.output_is_folder = false;
		converter.activate_as_sub_doc = true;
		converter.activate_by_pattern = true;
		converter.activation_document_type = "texts";
		converter.output_extension = ".txt";

		converter.activation_pattern = "*.pdf";
        found_converter = option_set["filters.loading"].length;
	}
    else
        converter = option_set["filters.loading"][found_converter].clone();

	converter.arguments = "$(input) $(output)";

    if (converter.application_path == "")
    {
		var path;
		if (Machine.system_name.match (/.*Win.*/) &&
			(path=ecmerge_converters_which("ptconverter" + ec_get_default_app_extension())) !== undefined)
		{
			converter.application_path = path;
		}
    }
    option_set["filters.loading"][found_converter] = converter;

	converter_name = "ABC PDF Converter (ProcessTest Group)"; // have a variable for that to be sure, find/new share the same name
	found_converter = ec_find_converter(option_set, converter_name);
    
    if (found_converter===undefined)
	{
		var converter = new UserProcessFilter;
		converter.name = converter_name;
		converter.output_is_folder = false;
		converter.activate_as_sub_doc = true;
		converter.activate_by_pattern = true;
		converter.activation_document_type = "texts";
		converter.output_extension = ".txt";

		converter.activation_pattern = "*.pdf";
        found_converter = option_set["filters.loading"].length;
	}
    else
        converter = option_set["filters.loading"][found_converter].clone();

	converter.arguments = "$(input) $(output) 5";
    
    if (converter.application_path == "")
    {
		var path;
		if (Machine.system_name.match (/.*Win.*/) &&
			(path=ecmerge_converters_which("abcpdf" + ec_get_default_app_extension())) !== undefined)
		{
			converter.application_path = path;
        }
    }
    option_set["filters.loading"][found_converter] = converter;
}

ec_converters_detectors.push(ec_pdf_converter);

//////////////////////////////////////////////////////////////////
// Properties converters
function ec_exiftool_converter(option_set)
{
	var converter_name = "ExifTool"; // have a variable for that to be sure, find/new share the same name
	
	var arguments = "--Directory --FileName --FilePermissions --FileModifyDate --ExifToolVersion $(input)";

    var found_converter = ec_find_converter(option_set, converter_name);

	if (found_converter===undefined)
	{
		converter = new UserProcessFilter;
		converter.name = converter_name;
		converter.output_is_folder = false;
		converter.activate_as_sub_doc = true;
		converter.activate_by_pattern = true;
		converter.activation_document_type = "texts";
		converter.output_extension = ".txt";

		converter.activation_pattern = "*.exe;*.dll;*.mp3;*.wav;*.avi;*.divx;*.ogg";
		
        found_converter = option_set["filters.loading"].length;
    }
	else
		converter = option_set["filters.loading"][found_converter].clone();

    if (converter.application_path == "")
    {
	    var path;
	    if ((path=ecmerge_converters_which("exiftool" + ec_get_default_app_extension())) !== undefined)
	    {
		    converter.application_path = ec_get_stdio_converter();
            converter.arguments = Machine.escape_for_command_line(path) + " no \"\" yes $(output) " + arguments
	    }
    }

	option_set["filters.loading"][found_converter] = converter;

}

ec_converters_detectors.push(ec_exiftool_converter);

//////////////////////////////////////////////////////////////////
// Built-in converters

// users should make a copy of the converter to tweak it and disable the 
//	built-in one if they are not OK with it
// also Xtract is prioritized over non-native converters, hence the insertions
//	with 'splice' and not 'push'
function ec_ecxtract_converter(option_set)
{
    var converter;
	var converter_name = "Elli Computing Xtract"; // have a variable for that to be sure, find/new share the same name
	
	var index = ec_find_converter(option_set, converter_name);
	
	if (index===undefined)
	{
		converter = new UserProcessFilter;
		converter.name = converter_name;
		option_set["filters.loading"].splice (0, 0, converter);
        index = 0;
    }

    converter = option_set["filters.loading"][index];

    // always overwrite everything: this is our converter it must work as expected
	converter.output_is_folder = false;
	converter.activate_by_pattern = true;
	converter.activation_document_type = "texts";
	converter.activate_as_sub_doc = true;
	converter.output_extension = ".txt"

	converter.activation_pattern = "*.sxw;*.docx;*.odt;*.rtf;*.doc";
	converter.arguments = "--format=text --serial-number=$(serial) -o $(output) $(input)"

	var ecxtract_cli_name = "clixtract" + ec_get_default_app_extension()

    // as long as we have a good path, force it
	var path = URL(executable_path).compose("../" + ecxtract_cli_name);
	if (VFS.dir(URL(executable_path).compose(".."), ecxtract_cli_name).length ||
		(path=Machine.which(ecxtract_cli_name)) !== undefined)
	{
		converter.application_path = path;
	}
}

ec_converters_detectors.push(ec_ecxtract_converter);

function ec_ecxtract_arch_converter(option_set)
{
	var converter_name = "Elli Computing Xtract (Archives)"; // have a variable for that to be sure, find/new share the same name
	
	var index = ec_find_converter(option_set, converter_name);
	
	if (index===undefined)
	{
		converter = new UserProcessFilter;
		converter.name = converter_name;
		// Xtract is not like other filters: we give it priority
		option_set["filters.loading"].splice (0, 0, converter);
        index = 0;
    }

    converter = option_set["filters.loading"][index];

	converter.output_is_folder = true;
	converter.activate_by_pattern = true;
	converter.activation_document_type = "folders";
	converter.output_extension = ""

	converter.activation_pattern = "*.zip";
	converter.arguments = "--format=folder --serial-number=$(serial) -o $(output) $(input)"

	var ecxtract_cli_name = "clixtract" + ec_get_default_app_extension()

	var path = URL(executable_path).compose("../" + ecxtract_cli_name);
		
	if (VFS.dir(URL(executable_path).compose(".."), ecxtract_cli_name).length ||
		(path=Machine.which(ecxtract_cli_name)) !== undefined)
	{
		converter.application_path = path;
	}
}

ec_converters_detectors.push(ec_ecxtract_arch_converter);

function ec_ecxtract_xml_arch_converter(option_set)
{
	// files known to be XML archives (i.e. all OOXML / MSOffice XML not yet
	//	supported by a native 'to text' converter in Xtract, but opening the
	//	ZIP is still cool)
	var converter_name = "Elli Computing Xtract (Xml Archives)"; // have a variable for that to be sure, find/new share the same name
	
	var index = ec_find_converter(option_set, converter_name);
	
	if (index===undefined)
	{
		converter = new UserProcessFilter;
		converter.name = converter_name;
		// Xtract is not like other filters: we give it priority
		option_set["filters.loading"].splice (0, 0, converter);
        index = 0;
    }

    converter = option_set["filters.loading"][index];

	converter.output_is_folder = true;
	converter.activate_by_pattern = true;
	converter.activation_document_type = "folders";
	converter.output_extension = ""

	converter.activation_pattern = "*.xlsx;*.xlsm;*.xltx;*.xltm;*.xlam;*.pptx;*.pptm;*.potx;*.potm;*.ppam;*.ppsx;*.ppsm;"+
		"*.odp;*.ods;*.odg;*.stc;*.sti;*.std;*.stw;*.sxc;*.sxd;*.sxi;*.sxm;*.sxg";
	converter.arguments = "--format=folder -A .zip --serial-number=$(serial) -o $(output) $(input)"

	var ecxtract_cli_name = "clixtract" + ec_get_default_app_extension()

	var path = URL(executable_path).compose("../" + ecxtract_cli_name);

	if (VFS.dir(URL(executable_path).compose(".."), ecxtract_cli_name).length ||
		(path=Machine.which(ecxtract_cli_name)) !== undefined)
	{
		converter.application_path = path;
	}
}

ec_converters_detectors.push(ec_ecxtract_xml_arch_converter);

function ec_ecxtract_app_arch_converter(option_set)
{
	// files known to be ZIP applications archives (i.e. like Java stuff in a ZIP)
	var converter_name = "Elli Computing Xtract (Applications Archives)"; // have a variable for that to be sure, find/new share the same name
	
	var index = ec_find_converter(option_set, converter_name);
	
	if (index===undefined)
	{
		converter = new UserProcessFilter;
		converter.name = converter_name;
		// Xtract is not like other filters: we give it priority
		option_set["filters.loading"].splice (0, 0, converter);
        index = 0;
    }

    converter = option_set["filters.loading"][index];

	converter.output_is_folder = true;
	converter.activate_by_pattern = true;
	converter.activation_document_type = "folders";
	converter.output_extension = ""

	converter.activation_pattern = "*.apk;*.jar;*.war;*.ear*;*.sar";
	converter.arguments = "--format=folder -A .zip --serial-number=$(serial) -o $(output) $(input)"

	var ecxtract_cli_name = "clixtract" + ec_get_default_app_extension()

	var path = URL(executable_path).compose("../" + ecxtract_cli_name);

	if (VFS.dir(URL(executable_path).compose(".."), ecxtract_cli_name).length ||
		(path=Machine.which(ecxtract_cli_name)) !== undefined)
	{
		converter.application_path = path;
	}
}

ec_converters_detectors.push(ec_ecxtract_app_arch_converter);

//////////////////////////////////////////////////////////////////
// Excel converters
function ec_excel_vbs_converter(option_set)
{
	var converter_name = "VBScript Excel converter"; // have a variable for that to be sure, find/new share the same name
	var found_converter = ec_find_converter(option_set, converter_name);
    
    if (found_converter===undefined)
	{
		converter = new UserProcessFilter;
		converter.name = converter_name;
		converter.output_is_folder = false;
		converter.activate_by_pattern = true;
		converter.activate_as_sub_doc = true;
		converter.activation_document_type = "texts";
		converter.output_extension = ".csv"

		converter.activation_pattern = "*.xls";

        found_converter = option_set["filters.loading"].length;
	}
    else
        converter = option_set["filters.loading"][found_converter].clone();

	var script_name = "converters/xl2csv.vbs"

	var script_path = URL(executable_path).compose("../" + script_name);
	converter.arguments = Machine.escape_for_command_line(script_path) + " $(input) $(output)"

    if (converter.application_path == "")
    {
		
		if (Machine.system_name.match(/.*Win.*/) &&
			Registry.get_subkeys_list("HKEY_LOCAL_MACHINE\\SOFTWARE\\Microsoft\\Office\\Excel"))
		{
			converter.application_path = "wscript";
		}
    }

    option_set["filters.loading"][found_converter] = converter;
}

ec_converters_detectors.push(ec_excel_vbs_converter);

//////////////////////////////////////////////////////////////////
// WordPerfect converters
function ec_libwpd_converter(option_set)
{
	ec_add_zfilter (option_set, "WordPerfect (libwpd)", "*.wpd", "wpd2text", false, true, "$(input)", ".txt", true)
}

ec_converters_detectors.push(ec_libwpd_converter);
