// (c) Elli Computing
// Name: complete-xml-tag.js
// Version: 1.2
// Purpose: complete the current tag, adding the matching close tag
//		if the current tag is still open, adds > then </name>
//		if the current tag already has its > and is not yet closed, just add the close tag
//		if the current tag is already closed, abandon
//		if the tag name is inside the array, of INLINE_TAGS, no specific indentation stuff is done
//		if the AUTO_INLINE flag is true, and the opening tag was not preceded only by blanks, no identation stuff is done
//		if identation stuff is done, return is added and cursor is moved after a new line indented once more than the tag itself

// History:
// 1.1 uses new 'document.information' member to determine actual language, fixes accept characters in tags (to include ., _, : and numbers)

// An INLINE_TAGS_PER_LG global array is initialised by this script, it is designed to contain inline tags per language as defined in the
//		language colouring file (syntaxcolouring.xml file)

var INLINE_TAGS_PER_LG = [];

macros["complete_xml_tag"] = function() {

	var INLINE_TAGS = [];
	var AUTO_INLINE = false;
	var CONTEXT_CHARS_MAX = 10000; // accepts to search that amount of characters before current position for start of current tag
	
	var currentview = current_frame.coordinator.current_view;
	var currentrole = currentview.role;
	
	function back_find (content, end_position, re_text)
	{	
		var start_position = Math.max(0, end_position - CONTEXT_CHARS_MAX);
		
		var start_match = new RegExp (re_text, 'g');
		start_match.lastIndex = end_position;
		start_match.endIndex = start_position;
		// log ("searching between: "+start_position + " and "+ end_position);
		return start_match.execBackward (content);
	}
	
	function find_inline_tag(name)
	{
		for (var i=0; i<INLINE_TAGS.length; ++i)
			if (name == INLINE_TAGS[i])
				return true;
		return false;
	}
	
	function find_tag (content, position)
	{
		var	name, // name of the tag
				found_at, // position where at was found
				indent, // should we indent at all
				indent_with, // with what to indent
				extra_line,	// should insert an extra-line with the cursor ready to input text
				end_tag,	// add the >
				move_after; // move after end of tag
		
		// we define a tag as a < then maybe ? or ! then a name then any number of anything but strings, ending with maybe / then maybe > 
		var found_tag_info = back_find (content, position, '<(/)?((?:\\w|[:_])(?:\\w|[:_.0-9])*)(?:(?:\\s+(?:(?:\\w|[:_])(?:\\w|[:_.0-9])*)(?:\\s*=\\s*(?:".*?"|\'.*?\'|[^\'">\\s]+))?)+\\s*|\\s*)(/?)(>?)');
		if (!found_tag_info)
		{
			log ("no tag");
			return undefined;
		}
		
		if (found_tag_info[1] == '/' ||
			found_tag_info[3] == '/')
		{
			log ("(auto-)closing tag found");
			return undefined;
		}
			
		found_at = found_tag_info.index;
		name = found_tag_info[2];
		end_tag = found_tag_info[4] == '';
				
		var line_start_pos = content.position_from_line(content.line_from_position (found_at));
		var line_start = content.get_text(new TextDocument.Selection(content, LinearRange(line_start_pos, found_at-line_start_pos)));

		// set if there are already characters inside the tag
		move_after = found_tag_info[4] == '>' && found_tag_info.endIndex != position && content.line_from_position(found_tag_info.endIndex) == content.line_from_position(position);

		// log ("line start at: "+line_start_pos + " up to " + found_at + ", containing: " + line_start);
		var line_start_wo_blanks = line_start.replace(/\b/g, '');
		// log ("line_start_wo_blanks: " + line_start_wo_blanks);
		indent = !find_inline_tag (name) && (!AUTO_INLINE || (line_start_wo_blanks == '')) && !move_after;
		
		if (indent && line_start_wo_blanks != '')
			indent_with = /^[\t ]*/.exec(line_start)[0];	// match the blanks at start of line
		else
			indent_with = indent ? line_start : undefined;

		extra_line = indent;
	
		return { name: name, position: found_at, indent: indent, indent_with: indent_with, extra_line: extra_line, end_tag: end_tag, move_after: move_after };
	}
	
	function complete_xml_tag ()
	{
		var content = currentdoc.content[currentrole];
		var position = currentview.clone_selection().range.start;
		var final_cursor_position = position;
		var found = find_tag (content, position);
		if (!found) return;
		
		if (found.end_tag) 
		{
			final_cursor_position = position = content.set_text (new TextDocument.Selection(content, LinearRange (position, 0)), '>');
		}
		
		if (found.extra_line)
		{
			final_cursor_position = position = content.set_text (new TextDocument.Selection(content, LinearRange (position, 0)), '\r\n' + found.indent_with + '\t');
		}
		
		if (found.indent)
		{
			position = content.set_text (new TextDocument.Selection(content, LinearRange (position, 0)), '\r\n' + found.indent_with);
		}
		
		var end_position = content.set_text (new TextDocument.Selection(content, LinearRange (position, 0)), '</' + found.name + '>');
		if (found.move_after)
			final_cursor_position = end_position;
		
		currentview.navigate (final_cursor_position);
		
	}
	
	var currentdoc = current_frame.coordinator.document;
	if (!(currentdoc instanceof TextDocument))
		return alert("works only on text documents");
	
	if (currentdoc.information) // since 2.2.101
		log ("using tags of language:" + currentdoc.information[currentrole].syntax_colouring_language);
	
	if (INLINE_TAGS_PER_LG && currentdoc.information && 
		INLINE_TAGS_PER_LG[currentdoc.information[currentrole].syntax_colouring_language])
	{
		INLINE_TAGS = INLINE_TAGS_PER_LG[currentdoc.information[currentrole].syntax_colouring_language];
	}

	currentdoc.transaction (complete_xml_tag);
}

macros["complete_xml_tag"].label = _("XML Tag Completion")
macros["complete_xml_tag"].short_cut = "Ctrl+SPACE";
macros["complete_xml_tag"].available_for_document_types = "texts";

INLINE_TAGS_PER_LG["Hypertext"] = [ "i", "b", "u", "a", "span", "sub", "sup", "strong", "em", "tt", "code", "li", "dl", "dd" ];
INLINE_TAGS_PER_LG["Active Server Pages (alternate)"] = INLINE_TAGS_PER_LG["Hypertext"];
